<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Models\SupportingDocs;
use App\Rules\JsonSchemaRule;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Http;
use App\Models\Order;
use App\Models\ApiUser;
use DB;
use Illuminate\Contracts\Validation\Rule;
use JsonSchema\Validator as JsonSchemaValidator;
use Mail;

class ApiOrderController extends Controller
{
    public function store(Request $request)
    {
        $accessToken = $request->header('Authorization');
        $token = str_replace('Bearer ', '', $accessToken);
        $user = ApiUser::where('api_token', hash('sha256', $token))->first();
        if ($user && (hash('sha256', $token) == $user->api_token)) {
            if(now() >  $user->token_expires_at) {
                return response()->json(['error' => 'Token Expired'], 401);
            }

            $jsonSchema = '';
            $jsonData = $request->json()->all();


            $apiTraficId = DB::table('api_traffic')->insertGetId([
                'mode' => 'Incoming',
                'api_status' => 'Received',
                'request' => json_encode($jsonData),
                'created_at' => now(),
            ]);

            if($request->input('OrderInfo.service_id') == 1) {
                $jsonSchema = storage_path("app/JSON_Schema/input_schema_flood.json");
            } elseif ($request->input('OrderInfo.service_id') == 2) {
                $jsonSchema = storage_path("app/JSON_Schema/input_schema.json");
            }

            $jsonData = json_decode(json_encode($jsonData));

            $validator = new JsonSchemaValidator();
            $validator->validate($jsonData, (object)['$ref' => 'file://' . $jsonSchema]);

            if ($validator->isValid()) {
                $client_id = $request->input('OrderInfo.client_id');
                $service_id = $request->input('OrderInfo.service_id');
                $client_order_id = $request->input('OrderInfo.client_order_id');

                $apiConfig = DB::table('api_configuration')->where('client_id', $client_id)->where('is_active', 1)->first();
                $client = DB::table('clients')->where('id', $client_id)->where('is_active', 1)->first();
                $service = DB::table('services')->where('id', $service_id)->where('is_active', 1)->first();

                if (empty($apiConfig) || empty($client) || empty($service)) {
                    $disabledEntities = [];

                    if (empty($apiConfig)) {
                        $disabledEntities[] = "API Connection";
                    }

                    if (empty($client)) {
                        $disabledEntities[] = "Client";
                    }

                    if (empty($service)) {
                        $disabledEntities[] = "Service";
                    }

                    $errorMessage = implode(', ', $disabledEntities) . " is disabled by the Administrator";
                    DB::table('api_traffic')->where('id', $apiTraficId)->update(['response' => $errorMessage, 'api_status' => 'Failed', 'responded_at' => now()]);
                    return response()->json(["Message" => $errorMessage], 422);
                }

                if(!empty($request->input('PropertyData.legal_document'))) {
                    $decodedData = base64_decode($request->input('PropertyData.legal_document'));
                    $filename = uniqid('legal_document_') . '.' . 'pdf';
                    \Storage::put('Legal_Document_Inputs/' . $filename, $decodedData);
                    $jsonData->PropertyData->legal_document = $filename;
                }

                $orderId = Order::insertGetId([
                    'input' => json_encode($jsonData),
                    'client_id' => $client_id,
                    'service_id' => $service_id,
                    'client_order_id' => $client_order_id,
                    'is_api_order' => 1,
                    'status' => 1
                ]);
                $OrderData = Order::leftJoin('status', 'status.id', '=', 'orders.status')
                            ->leftJoin('clients', 'clients.id', '=', 'orders.client_id')
                            ->leftJoin('services', 'services.id', '=', 'orders.service_id')
                            ->select(
                                'orders.id as OrderId',
                                'status.status as OrderStatus',
                                'clients.client_name as Client',
                                'services.service_name as Service',
                                'orders.created_at as OrderCreatedAt'
                            )->where('orders.id', $orderId)->first();

                DB::table('api_traffic')->where('id', $apiTraficId)->update(['response' => json_encode($OrderData), 'api_status' => 'Completed', 'responded_at' => now()]);
                return response()->json($OrderData);
            } else {
                $errors = $validator->getErrors();
                DB::table('api_traffic')->where('id', $apiTraficId)->update(['response' => json_encode($errors), 'api_status' => 'Failed', 'responded_at' => now()]);
                return response()->json(['error' => $errors], 422);
            }
        } else {
            return response()->json(['error' => 'Invalid token'], 401);
        }
    }

    public function OrderHandover($orderId, $statusId, $comments)
    {
        $order = Order::where('id', $orderId)->first();
        $apiurl = $apiusername = $apipassword = "";

        if (!empty($order)) {
            $client_id = $order->client_id;
            $apiConfig = DB::table('api_configuration')->where('client_id', $client_id)->where('is_active', 1)->first();
            if($apiConfig) {
                $configData = json_decode($apiConfig->config_data, true);
                $apiurl = $configData['url'];
                $apiusername = $configData['username'];
                $apipassword = $configData['password'];
            } else {
                return response()->json(['error' => 'API Configuration Not Found']);
            }

        } else {
            return response()->json(['error' => 'Order ID not found']);
        }

        $endpoint_url = $apiurl.'/api';

        try
        {
            $tokenData = ApiUser::where('id', 1)
                ->where('client_token_expires_at', '>', date('Y-m-d H:i:s', strtotime('+1 minutes')))->first();

            if(empty($tokenData)) {
                $response = Http::timeout(60)
                                ->post($endpoint_url. '/login', [
                                    'username' => $apiusername,
                                    'password' => $apipassword,
                                ]);
                $responseData = json_decode($response->body(), true);
                ApiUser::where('id', 1)->update(['client_token' => $responseData['token'], 'client_token_expires_at' => date("Y-m-d H:i:s", strtotime($responseData['expires_at']))]);
                $accessToken = $responseData['token'];
            } else {
                $accessToken = $tokenData->client_token;
            }

            if (!empty($order)) {
                $serviceFolder = $order->service_id == 1 ? 'SFHDF_Form_Output' : 'LV_Form_Output';
                $filePath = storage_path("app/$serviceFolder/{$order->id}");

                $fileNames = ['Merged_Certificate.pdf', 'Certificate.pdf'];

                foreach ($fileNames as $fileName) {
                    if (File::exists("$filePath/$fileName")) {
                        $fileContent = base64_encode(file_get_contents("$filePath/$fileName"));
                        break;
                    }
                }
            }
            $request_data = "";
            if($statusId == 8 || $statusId == 9) {
                $request_data = [
                    'client_order_id' => $order->client_order_id,
                    'status_id' => $statusId,
                    'comments' => $comments,
                    'file' => $fileContent,
                    'json_response' => $order->output,
                    'api_user_id' => 1
                ];
            } else {
                $request_data = [
                    'client_order_id' => $order->client_order_id,
                    'status_id' => $statusId,
                    'comments' => $comments,
                    'api_user_id' => 1
                ];
            }

            $apiTraficId = DB::table('api_traffic')->insertGetId([
                'mode' => 'Outgoing',
                'api_status' => 'Requested',
                'request' => json_encode($request_data),
                'created_at' => now(),
            ]);

            $response = Http::withHeaders([
                'Authorization' => 'Bearer ' . $accessToken,
            ])->post($endpoint_url.'/status', $request_data, ['Content-Type' => 'application/json']);

            $responseData = $response->json();

            if ($response->status() == 401 || $response->status() != 200) {
                $response = Http::timeout(60)
                ->post($endpoint_url. '/login', [
                    'username' => $apiusername,
                    'password' => $apipassword,
                ]);
                $responseData = json_decode($response->body(), true);
                ApiUser::where('id', 1)->update(['client_token' => $responseData['token'], 'client_token_expires_at' => date("Y-m-d H:i:s", strtotime($responseData['expires_at']))]);
                $accessToken = $responseData['token'];

                $response = Http::withHeaders([
                    'Authorization' => 'Bearer ' . $accessToken,
                ])->post($endpoint_url.'/status', $request_data, ['Content-Type' => 'application/json']);

                $responseData = $response->json();
            }

            if ($response->status() == 200) {
                if(!empty($responseData['status'])) {
                    if ($responseData['status'] == 200) {
                        DB::table('api_traffic')->where('id', $apiTraficId)->update([
                            'response' => json_encode($responseData),
                            'api_status' => 'Completed',
                            'responded_at' => now(),
                        ]);
                    } else {
                        DB::table('api_traffic')->where('id', $apiTraficId)->update([
                            'response' => json_encode($responseData),
                            'api_status' => 'Failed',
                            'responded_at' => now(),
                        ]);

                        $content = "<p>Hello Team,</p>" .
                            "<p>We doesn't get any response while processing Client Oder ID: {$order->client_order_id}</p>";
                        Mail::html($content, function ($message) {
                            $message->to('arunsakthian.mr@stellaripl.com')
                                    ->cc(['ajith.km@stellaripl.com', 'rajalakshmimani@stellaripl.com'])
                                    ->subject("LenderOMS API Request Failure");
                        });
                    }
                }
            } else {
                $responseError = "API Request Failed with Error";
                DB::table('api_traffic')->where('id', $apiTraficId)->update([
                    'response' => json_encode($responseError),
                    'api_status' => 'Failed',
                    'responded_at' => now(),
                ]);
                $content = "<p>Hello Team,</p>" .
                "<p>We doesn't get any response while processing Client Oder ID: {$order->client_order_id}</p>";
                Mail::html($content, function ($message) {
                    $message->to('arunsakthian.mr@stellaripl.com')
                            ->cc(['ajith.km@stellaripl.com', 'rajalakshmimani@stellaripl.com'])
                            ->subject("LenderOMS API Request Failure");
                });
                return response()->json(['error' => $responseError]);
            }

            return response()->json(['success' => $responseData]);
        } catch(Exception $e) {
            $responseError = "API Request Failed with Error";
            DB::table('api_traffic')->where('id', $apiTraficId)->update([
                'response' => json_encode($responseError),
                'api_status' => 'Failed',
                'responded_at' => now(),
            ]);
            "<p>We doesn't get any response while processing Client Oder ID: {$order->client_order_id}</p>";
            Mail::html($content, function ($message) {
                $message->to('arunsakthian.mr@stellaripl.com')
                        ->cc(['ajith.km@stellaripl.com', 'rajalakshmimani@stellaripl.com'])
                        ->subject("LenderOMS API Request Failure");
            });
            return response()->json(['error' => $responseError]);
        }

        return response()->json(["success" => $responseData ?? ""]);
    }

    public function clientupdate(Request $request)
    {
        $accessToken = $request->header('Authorization');
        $token = str_replace('Bearer ', '', $accessToken);
        $user = ApiUser::where('api_token', hash('sha256', $token))->first();
        if ($user && (hash('sha256', $token) == $user->api_token)) {
            if(now() >  $user->token_expires_at) {
                return response()->json(['error' => 'Token Expired'], 401);
            }

            $jsonData = $request->json()->all();
            $filename = null;

            $apiTraficId = DB::table('api_traffic')->insertGetId([
                'mode' => 'Incoming',
                'api_status' => 'Received',
                'request' => json_encode($jsonData),
                'created_at' => now(),
            ]);

            if (!empty($jsonData['order_id'])) {
                $Order = DB::table('orders')->where('id', $jsonData['order_id'])->first();

                if (!empty($request->input('file'))) {
                    $decodedData = base64_decode($request->input('file'));
                    $filename = uniqid() . '.pdf';

                    // Assuming $OrderData is defined somewhere in your code
                    if ($Order) {
                        $basepath = $Order->service_id == 1 ? 'SFHDF_Form_Output' : 'LV_Form_Output';
                        \Storage::put("$basepath/{$Order->id}/$filename", $decodedData);

                        SupportingDocs::insertGetId([
                            'order_id' => $Order->id,
                            'pdf_file' => $filename,
                            'created_at' => now(),
                        ]);
                    }
                }
            } else {
                $response = ['OrderId' => $jsonData['order_id'], 'message' => "Order ID not found in Request"];
                DB::table('api_traffic')->where('id', $apiTraficId)->update(['response' => json_encode($response), 'api_status' => 'Failed', 'responded_at' => now()]);
                return response()->json($response);
            }

            $latestRecord = DB::table('orders_status_history')
            ->where('order_id', $jsonData['order_id'])
            ->where('status_id', $jsonData['status_id'])
            ->latest('updated_at')
            ->first();

            // Update the latest matching record
            if ($latestRecord) {
                DB::table('orders_status_history')
                    ->where('id', $latestRecord->id) // Assuming there's an 'id' column
                    ->update(['client_comments' => $jsonData['comments'], 'attachment' => $filename ?? null]);

                DB::table('orders')
                    ->where('id', $latestRecord->order_id)
                    ->update(['status' => 6]);

                DB::table('orders_status_history')
                    ->insert(['order_id' => $latestRecord->order_id ,'status_id' => 6, 'comments' => "Received Clarification from Client"]);
            } else {
                $response = ['OrderId' => $jsonData['order_id'], 'message' => "No Records found with this Order ID"];
                DB::table('api_traffic')->where('id', $apiTraficId)->update(['response' => json_encode($response), 'api_status' => 'Failed', 'responded_at' => now()]);
                return response()->json($response);
            }

            $response = ['OrderId' => $jsonData['order_id'], 'message' => "Inputs added successfully"];
            DB::table('api_traffic')->where('id', $apiTraficId)->update(['response' => json_encode($response), 'api_status' => 'Completed', 'responded_at' => now()]);
            return response()->json($response);
        } else {
        return response()->json(['error' => 'Invalid token'], 401);
        }
    }
}
