<?php

namespace App\Http\Controllers;

use PDF;
use App\Models\Order;
use App\Models\SupportingDocs;
use mikehaertl\pdftk\Pdf as pdfTk;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;

class FloodCertPDFController extends Controller
{
    public function fillPDF($orderId) {
        $templatePdf = public_path('SFHDF_Form_2023/fema_form-ff-206-fy-21-116_112022.pdf');
        $coverTemplatePdf = public_path('SFHDF_Form_2023/flood_certificate_cover.pdf');
        $directoryPath = "SFHDF_Form_Output/{$orderId}";
        $filePath = storage_path("app/{$directoryPath}");
        $outputFilePath = storage_path("app/{$directoryPath}/Certificate.pdf");
        $coverOutputFilePath = storage_path("app/{$directoryPath}/Flood_Notice.pdf");

        if (!Storage::exists($directoryPath)) {
            Storage::makeDirectory($directoryPath, 0777, true);
        } else {
            chmod(storage_path("app/{$directoryPath}"), 0777);
        }

        $pdf = new pdfTk($templatePdf);
        $orderData = Order::where('id', $orderId)->first();
        $data = json_decode($orderData->output, true);
        // Combine multiple fields into a single field for PREPARER'S INFORMATION
        $preparerName = $data['flood_cert_data']['preparer_information']['name'];
        $preparerAddress = $data['flood_cert_data']['preparer_information']['address'];
        $preparerNumber = $data['flood_cert_data']['preparer_information']['number'];
        $preparerServiceNumber = $data['flood_cert_data']['preparer_information']['service_number'];

        $lenderName = $data['flood_cert_data']['loan_information']['lender_name'];
        $lenderAddress = $data['flood_cert_data']['loan_information']['lender_address'];
        $lenderInformation = "$lenderName\n$lenderAddress";

        $preparerInformation = '';
        if (!empty($preparerName)) {
            $preparerInformation .= "$preparerName";
        }
        if (!empty($preparerAddress)) {
            $preparerInformation .= "\n$preparerAddress";
        }
        if (!empty($preparerNumber)) {
            $preparerInformation .= "\nContact: $preparerNumber";
        }
        if (!empty($preparerServiceNumber)) {
            $preparerInformation .= "\nNumber: $preparerServiceNumber";
        }
        $preparerInformation = trim($preparerInformation);

        function formatDate($date) {
            return ($date ? date('m/d/Y', strtotime($date)) : '');
        }

        if($data['flood_cert_data']['comments']['comments'] == "Life of Loan Tracking will be performed on this property and is transferable." ||
            $data['flood_cert_data']['comments']['comments'] == "Single Determination. No tracking will be performed on this property.") {
                $comments = "\n\n\n\n".$data['flood_cert_data']['comments']['comments'];
        } else {
            $comments = "\n".$data['flood_cert_data']['comments']['comments'];
        }

        // Map data to PDF fields
        $pdfFields = [
            'form1[0].#subform[3].TextField1[0]' => $lenderInformation,
            'form1[0].#subform[3].TextField2[0]' => "\n".$data['flood_cert_data']['loan_information']['collateral_desc'],
            'form1[0].#subform[3].TextField3[0]' => $data['flood_cert_data']['loan_information']['loan_identifier'],
            'form1[0].#subform[3].TextField16[0]' => $data['flood_cert_data']['loan_information']['lender_id'],
            'form1[0].#subform[3].TextField4[0]' => $data['flood_cert_data']['loan_information']['amount_of_flood_insurance'],
            'form1[0].#subform[3].TextField5[0]' => strtoupper($data['flood_cert_data']['jurisdiction']['community_name']),
            'form1[0].#subform[3].TextField6[0]' => strtoupper($data['flood_cert_data']['jurisdiction']['county']),
            'form1[0].#subform[3].TextField15[0]' => strtoupper($data['flood_cert_data']['jurisdiction']['state']),
            'form1[0].#subform[3].NumericField2[0]' => strtoupper($data['flood_cert_data']['jurisdiction']['community_num']),
            'form1[0].#subform[3].TextField8[0]' => strtoupper($data['flood_cert_data']['flood_insurance']['map_num']),
            'form1[0].#subform[3].TextField9[0]' => formatDate($data['flood_cert_data']['flood_insurance']['map_panel']),
            'form1[0].#subform[3].RadioButtonList[0]' => strtoupper($data['flood_cert_data']['flood_insurance']['map_change']),
            'form1[0].#subform[3].DateField1[0]' => formatDate($data['flood_cert_data']['flood_insurance']['date']),
            'form1[0].#subform[3].TextField10[0]' => strtoupper($data['flood_cert_data']['flood_insurance']['flood_zone']),
            'form1[0].#subform[3].TextField11[0]' => strtoupper($data['flood_cert_data']['flood_insurance']['map']),
            'form1[0].#subform[3].TextField12[0]' => strtoupper($data['flood_cert_data']['flood_insurance']['case_no']),
            'form1[0].#subform[3].CheckBox3[0]' => $data['flood_cert_data']['ffi_availability']['insurance_available'] ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox4[0]' => $data['flood_cert_data']['ffi_availability']['regular_program'] ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox5[0]' => $data['flood_cert_data']['ffi_availability']['emergency_program'] ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox6[0]' => $data['flood_cert_data']['ffi_availability']['insurance_not_available'] ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox7[0]' => $data['flood_cert_data']['ffi_availability']['federal_flood_insurance'] ? '1' : 'Off',
            'form1[0].#subform[3].DateField3[0]' => formatDate($data['flood_cert_data']['ffi_availability']['designation_date']),
            'form1[0].#subform[3].CheckBox8[0]' => ($data['flood_cert_data']['determination']['in_special_flood_hazard_area'] == "1") ? '1' : 'Off',
            'form1[0].#subform[3].CheckBox9[0]' => ($data['flood_cert_data']['determination']['in_special_flood_hazard_area'] == "off") ? '1' : 'Off',
            'form1[0].#subform[3].TextField13[0]' => $preparerInformation,
            'form1[0].#subform[3].TextField14[0]' => $comments,
            'form1[0].#subform[3].DateField2[0]' => formatDate($data['flood_cert_data']['preparer_information']['date_of_determination']),
        ];

        $result = $pdf->fillForm($pdfFields)->flatten()->saveAs($outputFilePath);

        if($data['flood_cert_data']['determination']['in_special_flood_hazard_area'] == "1") {
            if(strlen($data['flood_cert_data']['jurisdiction']['community_num']) == 6) {
                $pdf_cover = new pdfTk($coverTemplatePdf);

                $pdfFields = [
                    'community_number' => $data['flood_cert_data']['jurisdiction']['community_num']
                ];

                $result = $pdf_cover->fillForm($pdfFields)->flatten()->saveAs($coverOutputFilePath);

                $pdfFiles = [$outputFilePath, $coverOutputFilePath];

                $gs = (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') ? resource_path('gs9.55.0/bin/gswin64c.exe') : '/usr/bin/gs';
                $outputFile = "{$filePath}/Certificate_withcover.pdf";
                $command = "{$gs} -sDEVICE=pdfwrite -dNOPAUSE -dBATCH -dSAFER -sOutputFile={$outputFile}";

                foreach ($pdfFiles as $file) {
                    $command .= ' ' . escapeshellarg($file);
                }

                $descriptorspec = [
                    0 => ['pipe', 'r'],
                    1 => ['pipe', 'w'],
                    2 => ['pipe', 'w'],
                ];

                $process = proc_open($command, $descriptorspec, $pipes);

                if (is_resource($process)) {
                    fclose($pipes[0]);
                    fclose($pipes[1]);
                    fclose($pipes[2]);
                    while ($status = proc_get_status($process)) {
                        if (!$status['running']) {
                            $exitCode = $status['exitcode'];
                            break;
                        }
                        usleep(100000);
                    }
                    $endFileCount = count(File::files($filePath));

                    if($exitCode === 0) {
                        if(substr(PHP_OS, 0, 3) != 'WIN') {
                            $this->changeFolderPermissions($directoryPath, 0777);
                        }
                        foreach ($pdfFiles as $file) {
                            unlink($file);
                        }

                        if(rename($outputFile, $outputFilePath)) {
                            $responseData = ['success' => "PDF Generated Successfully"];
                        } else {
                            $responseData = ['error' => "PDF Renaming Failed"];
                        }
                    } else {
                        $responseData = ['error' => "PDF Not Generated"];
                    }

                    proc_close($process);

                    return response()->json($responseData);
                } else {
                    return response()->json(['error' => "Failed to execute the shell command"]);
                }
            }
        }

        $msg = '';

        if ($result === false)
        {
            $msg = $pdf->getError();
        } else {
            $msg = "Form Filled Successfully";
        }

        if(substr(PHP_OS, 0, 3) != 'WIN') {
            $this->changeFolderPermissions($directoryPath, 0777);
        }

        return response()->json($msg);
    }

    protected function changeFolderPermissions($path, $permissions)
    {
        if (file_exists($path) || is_dir($path)) {
            chmod($path, $permissions);
            $items = scandir($path);

            foreach ($items as $item) {
                if ($item != '.' && $item != '..') {
                    $itemPath = $path . '/' . $item;
                    if (is_dir($itemPath)) {
                        $this->changeFolderPermissions($itemPath, $permissions);
                    } else {
                        chmod($itemPath, $permissions);
                    }
                }
            }
        }
    }
}
