<?php

namespace App\Http\Controllers;

use App\Models\Company;
use App\Models\ClientType;
use App\Models\User;
use App\Models\State;
use App\Models\County;
use App\Models\Service;
use App\Models\OutputType;
use App\Models\Order;
use App\Models\Client;
use App\Models\Status;
use App\Models\UserType;
use App\Models\ServiceUserMapping;
use App\Models\ApiConfig;
use DB;
use Hash;
use Session;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class SettingController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function setting(Request $request)
    {
        if ($request->is('settings/users')) {
            if (Auth::user()->hasRole('Super Admin')) {
                $usersData = User::with('usertypes:id,user_types', 'companies')->whereIn('user_type_id', [2,3,4])->get();
                $userTypes = UserType::whereIn('id', [2,3,4])->get();
            } elseif (Auth::user()->hasRole('Admin')) {
                $usersData = User::with(['usertypes:id,user_types', 'companies'])->whereIn('user_type_id', [3,4])->where('company_id', session('company_id'))->get();
                $userTypes = UserType::whereIn('id', [3,4])->get();
            }

            $companyList = Company::where('is_active', 1)->get();

            return view('app.settings.users', compact('usersData', 'userTypes', 'companyList'));
        } else if ($request->is('settings/company')) {
            $companyData = Company::with('states:id,state_name,short_code','counties:id,county_name')->get();
            $stateList = State::select('id','short_code')->get();
            $countyList = County::select('id','county_name')->get();

            return view('app.settings.company', compact('companyData','stateList','countyList'));
        } else if ($request->is('settings/services')) {
            if (Auth::user()->hasRole('Super Admin')) {
                $serviceData = Service::with('clients', 'companies')->get();
            } elseif (Auth::user()->hasRole('Admin')) {
                $serviceData = Service::with('clients', 'companies')->where('company_id', Session::get('company_id'))->get();
            }
            $outputList = Service::all();
            $companyList = Company::where('is_active', 1)->get();

            return view('app.settings.services', compact('serviceData','outputList', 'companyList'));
        }else if ($request->is('settings/api')) {
            if (Auth::user()->hasRole('Super Admin')) {
                $apiConfigurations = ApiConfig::all();
                $serviceList = Service::all();
                // $clientList = Client::where('is_active', 1)->get();
                $clientList = Client::all();
                // dd($clientList);
                return view('app.settings.api_config', compact('apiConfigurations', 'serviceList', 'clientList'));
            }
        }

         elseif ($request->is('settings/status')) {
            $statusData = Status::all();

            return view('app.settings.status', compact('statusData'));
        } elseif ($request->is('settings/client')) {
            $companyData = Company::select('id', 'company_name', 'is_active')
                            ->where('is_active', 1)
                            ->get();
            $clientTypeData = ClientType::where('is_active', 1)->get();
            $stateList = DB::table('state')
                    ->select('id', 'state_name', 'short_code', 'is_active', 'eld_date', 'eld_month')
                    ->get();
            $countyList = DB::table('county')
                    ->select('id', 'stateId', 'county_name', 'agency_code', 'fips_code', 'calling', 'online_script', 'is_active')
                    ->where('is_active', 1) // Assuming you want to filter active records
                    ->get();
            $clientData = Client::with('companies', 'clientTypes')->get();
            return view('app.settings.client', compact('companyData','clientTypeData','stateList','clientData','countyList'));
        }
        elseif ($request->is('settings/order_delete/e9bac7fba602f592bb54eba0453001551c15d20300038fcf43c9d0a385bc9057')) {

            $orders = Order::with('statusTypes', 'clients', 'services', 'assignedUsers', 'updatedByUsers')
                ->get();

            return view('app.settings.order_delete', compact('orders'));
        }

        else {
            if (Auth::user()->hasRole('Super Admin')) {
                $companyData = Company::with('states:id,state_name,short_code','counties:id,county_name')->get();
                $stateList = State::select('id','short_code')->get();
                $countyList = County::select('id','county_name')->get();

                return view('app.settings.company', compact('companyData','stateList','countyList'));
            } elseif (Auth::user()->hasRole('Admin')) {
                $usersData = User::with(['usertypes:id,user_types', 'companies'])->whereIn('user_type_id', [3,4])->where('company_id', session('company_id'))->get();
                $userTypes = UserType::whereIn('id', [3,4])->get();

                $companyList = Company::where('is_active', 1)->get();

                return view('app.settings.users', compact('usersData', 'userTypes', 'companyList'));
            }
        }
    }

    public function getCounty(Request $request)
    {
        $getCounty['county'] = County::select('id','stateId','county_name')->where('stateId',$request->state_id)->get();

        return response()->json($getCounty);
    }

    public function getClients(Request $request)
    {
        $getClients['clients'] = Client::select('id', 'company_id', 'client_name')->where('company_id', $request->company_id)->where('is_active', 1)->get();

        return response()->json($getClients);
    }

    public function addCompany(Request $request)
    {
        $request->validate([
            'company_name' => 'required|unique:companies,company_name',
            'contact' => 'nullable|unique:companies,contact',
            'email' => 'nullable|unique:companies,email',
        ]);
        $input = $request->all();

		$isactive = (isset($input['is_active']))? 1 : 0;
        $check_company = Company::where('company_name', '=', $input['company_name'])->first();
        if(isset($check_company) && !empty($check_company) && ($check_company['company_name']==$input['company_name'])){
            return response()->json(['data'=>'error','msg'=>'Company Name Already Exists!']);
        } else {
            if($request->file('photo'))
            {
                $file = $request->file('photo');
				$extension = strtoupper($file->getClientOriginalExtension());
				if($extension=="PNG" || $extension=="JPEG" || $extension=="JPG" || $extension=="GIF" || $extension=="SVG"){
                $photo = base64_encode(file_get_contents($file->getRealPath()));
                $photo_type = $file->getMimeType();
				} else {
					return response()->json(['data'=>'error','msg'=>'Only Image Allowed!']);
				}
            } else {
                $photo = NULL;
                $photo_type = NULL;
            }
            // dd($isactive);
            $input['state'] = isset($request->state) ? $request->state : NULL;
            $input['county'] = isset($request->county) ? $request->county : NULL;
            $companyData = [
                'company_name' =>$input['company_name'],
                'address' => $input['address'],
                'city' => $input['city'],
                'state' => $input['state'],
                'county' => $input['county'],
                'contact' =>$input['contact'],
                'zip'=> $input['zipcode'],
                'logo'=> $photo,
				'email'=> $input['email'],
                'is_active' => $isactive
            ];
            if(Company::insert($companyData)){
                return response()->json(['data'=>'success','msg'=>'Company Added Successfully!']);
            } else {
                return response()->json(['data'=>'error','msg'=>$validator->errors()->all()]);
            }
        }
    }

    public function edit_company(Request $request)
    {
        $companyDetail = Company::where('id',$request->id)->first();
        return response()->json($companyDetail);
    }

    public function updateCompany(Request $request){
		$request->validate([
            'company_name' => 'required',
            'contact' => 'nullable',
            'email' => 'nullable',
        ]);
        $input = $request->all();
        $input['state'] = isset($request->state) ? $request->state : NULL;
        $input['county'] = isset($request->county) ? $request->county : NULL;
		$file = $request->file('photo');
		$isactive = (isset($input['is_active']))? 1 : 0;
		if($file){
                $extension = strtoupper($file->getClientOriginalExtension());
                if($extension=="PNG" || $extension=="JPEG" || $extension=="JPG" || $extension=="GIF" || $extension=="SVG"){
                $photo = base64_encode(file_get_contents($file->getRealPath()));
                $photo_type = $file->getMimeType();
			}
            else
            {
				return response()->json(['data'=>'error','msg'=>'Only Image Allowed!']);
			}
            $companyData = [
                'company_name' =>$input['company_name'],
                'address' => $input['address'],
                'city' => $input['city'],
                'state' => $input['state'],
                'county' => $input['county'],
                'contact' =>$input['contact'],
                'zip' => $input['zipcode'],
                'logo' => $photo,
				'email'=>$input['email'],
                'is_active' =>$isactive
            ];
		} else {
			$companyData = [
                'company_name' =>$input['company_name'],
                'address' => $input['address'],
                'city' => $input['city'],
                'state' => $input['state'],
                'county' => $input['county'],
                'contact' =>$input['contact'],
                'zip'=>$input['zipcode'],
				'email'=>$input['email'],
                'is_active' =>$isactive
            ];
		}
		$res = Company::where('id', $input['id'])->update($companyData);
		if($res==1) {
			return response()->json(['data'=>'success','msg'=>'Company Updated Successfully!']);
		}else if($res==0){
			return response()->json(['data'=>'success','msg'=>'No Changes To Update!']);
		}else {
			return response()->json(['data'=>'error','msg'=>$validator->errors()->all()]);
		}
    }

    public function companyStatus($companyId) {
        $company = Company::find($companyId);
        if ($company->is_active == 1) {
            Company::where('id', $companyId)->update(['is_active' => 0]);
        } else {
            Company::where('id', $companyId)->update(['is_active' => 1]);
        }

        return redirect()->back()->with('success', "Status Changed Successfully!");
    }

    //Users
    public function addUsers(Request $request)
    {
        $request->validate([
            'first_name' => 'required',
            'email' => 'nullable|unique:users,email,'.$request->id,
        ]);

        $input = $request->all();

        $isactive = $request->has('is_active') ? 1 : 0;
        $check_users = User::where('first_name', '=', $input['first_name'])->first();

        if(isset($check_users) && !empty($check_users)){
            $request->validate([
                'first_name' => 'required',
                'last_name' => 'required',
                'email' => 'nullable|unique:users,email,'.$request->id,
            ]);
            $check_users = User::where('first_name', '=', $input['first_name'])->where('last_name', '=', $input['last_name'])->first();
            if(isset($check_users) && !empty($check_users)){
                return response()->json(['data'=>'error','msg'=>'Users Already Exists!']);
            }
        }
        $company_id = $user_type_id = NULL;
        if(Auth::user()->hasRole('Super Admin')) {
            if($input['user_type_id'] == 2 || $input['user_type_id'] == 3 || $input['user_type_id'] == 4) {
                $request->validate([
                    'company_id' => 'required',
                ]);
                $company_id = $input['company_id'];
            }
            $user_type_id = $input['user_type_id'];
        } elseif(Auth::user()->hasRole('Admin')) {
            $company_id = Session::get('company_id');
            $user_type_id = $input['user_type_id'];
        }

        $usersData = [
            'company_id' => $company_id,
            'user_type_id' => $user_type_id,
            'first_name' => $input['first_name'],
            'last_name' => $input['last_name'],
            'email' => $input['email'],
            'password' => Hash::make($input['password']),
            'is_active' => $isactive,
        ];

        $userId = User::insertGetId($usersData);

        if (isset($userId)) {
            $user = User::find($userId);
            $user->assignRole($request->input('user_type_id'));

            return response()->json(['data' => 'success', 'msg' => 'Users Added Successfully!']);
        } else {
            return response()->json(['data' => 'error', 'msg' => $validator->errors()->all()]);
        }

    }

    public function edit_user(Request $request)
    {
        $id = $request->id;
        $userDetail = User::with('companies')->where('id', $id)->first();

        return response()->json($userDetail);
    }

    public function userStatus($userid)
    {
        $users = User::find($userid);
        if ($users->is_active == 1) {
            User::where('id', $userid)->update(['is_active' => 0]);
        } else {
            User::where('id', $userid)->update(['is_active' => 1]);
        }

        return redirect()->back()->with('success', 'Status Changed Successfully!');
    }

    public function updateUsers(Request $request)
    {

        $request->validate([
            'first_name' => 'required',
            'email' => 'nullable',
        ]);

        $input = $request->all();
        $isactive = (isset($input['is_active'])) ? 1 : 0;

        $company_id = $user_type_id = NULL;

        if(Auth::user()->hasRole('Super Admin')) {
            if($input['user_type_id'] == 2 || $input['user_type_id'] == 3 || $input['user_type_id'] == 4) {
                $request->validate([
                    'company_id' => 'required',
                ]);
                $company_id = $input['company_id'];
            }
            $user_type_id = $input['user_type_id'];
        } elseif(Auth::user()->hasRole('Admin')) {
            $company_id = Session::get('company_id');
            $user_type_id = $input['user_type_id'];
        }

        $usersData = [
            'company_id' => $company_id,
            'user_type_id' => $user_type_id,
            'first_name' => $input['first_name'],
            'last_name' => $input['last_name'],
            'email' => $input['email'],
            'password' => $input['password'],
            'is_active' => $isactive,
        ];

        $checkPass = User::where('id', $input['user_id'])->first();
        if ($checkPass->password != $input['password']) {
            $usersData['password'] = Hash::make($input['password']);
        }

        $res = User::where('id', $input['user_id'])->update($usersData);
        if ($res == 1) {
            DB::table('model_has_roles')->where('model_id', $input['user_id'])->delete();
            $user = User::find($input['user_id']);
            $user->assignRole($request->input('user_type_id'));

            return response()->json(['data' => 'success', 'msg' => 'Users Updated Successfully!']);
        } elseif ($res == 0) {
            return response()->json(['data' => 'success', 'msg' => 'No Changes To Update!']);
        } else {
            return response()->json(['data' => 'error', 'msg' => $validator->errors()->all()]);
        }
    }

    // Services
    public function addservice(Request $request)
    {
        $request->validate([
            'service_name' => 'required|unique:services,service_name',
            'short_code' => 'required',
            'description' => 'nullable',
            'company_id' => 'required',
        ]);

        $input = $request->all();

		$isactive = (isset($input['is_active']))? 1 : 0;
        $check_service = Service::where('service_name', '=', $input['service_name'])->first();
        if(isset($check_service) && !empty($check_service) && ($check_service['service_name']==$input['service_name'])){
            return response()->json(['data'=>'error','msg'=>'Service Name Already Exists!']);
        }
        else {
            $input['service_name'] = isset($request->service_name) ? $request->service_name : NULL;

            $serviceData = [
                'service_name' =>$input['service_name'],
                'company_id' =>$input['company_id'],
                'short_code' => $input['short_code'],
                'description' => $input['description'],
                'is_active' =>$isactive
            ];
            if(Service::insert($serviceData)){
                return response()->json(['data'=>'success','msg'=>'Service Added Successfully!']);
            } else {
                return response()->json(['data'=>'error','msg'=>$validator->errors()->all()]);
            }
        }
    }

    public function edit_service(Request $request)
    {
        $serviceDetail = Service::where('id',$request->id)->first();
        return response()->json($serviceDetail);
    }

    public function updateservice(Request $request)
    {
        $request->validate([
            'service_name' => 'required',
            'short_code' => 'required',
            'description' => 'nullable',
            'company_id' => 'required',
        ]);

        $input = $request->all();
        $input['output_type'] = isset($request->output_type) ? $request->output_type : null;
        $isactive = isset($input['is_active']) ? 1 : 0;

        $serviceData = [
            'service_name' => $input['service_name'],
            'company_id' =>$input['company_id'],
            'short_code' => $input['short_code'],
            'description' => $input['description'],
            'is_active' => $isactive,
        ];

        $res = Service::where('id', $input['id'])->update($serviceData);

        if ($res > 0) {
            return response()->json(['data' => 'success', 'msg' => 'Service Updated Successfully!']);
        } elseif ($res === 0) {
            return response()->json(['data' => 'success', 'msg' => 'No Changes To Update!']);
        } else {
            return response()->json(['data' => 'error', 'msg' => 'Failed to update service.']);
        }
    }


    public function serviceStatus($serviceId) {
        $service = Service::find($serviceId);
        if ($service->is_active == 1) {
            Service::where('id', $serviceId)->update(['is_active' => 0]);
        } else {
            Service::where('id', $serviceId)->update(['is_active' => 1]);
        }

        return redirect()->back()->with('success', "Status Changed Successfully!");
    }


     // Client

     public function addClient(Request $request)
     {
         $request->validate([
             'client_name' => 'required|unique:clients,client_name,NULL,id,company_id,' . $request->company_id,
         ]);

         $input = $request->all();
         $isactive = isset($input['is_active']) ? 1 : 0;

         $checkClient = Client::where('client_name', $input['client_name'])
             ->where('company_id', $input['company_id'])
             ->first();

         if ($checkClient) {
             return response()->json(['data' => 'error', 'msg' => 'Client name already exists for this company.']);
         }

         $logoPath = null;
         if ($request->hasFile('logo')) {
             $logo = $request->file('logo');
             $logoPath = 'Logo/' . $logo->getClientOriginalName();
             $logo->storeAs('public', $logoPath);
         }

         $clientData = [
             'company_id' => $input['company_id'],
             'client_name' => $input['client_name'],
             'client_type_id' => $input['client_type_id'],
             'address_line1' => $input['address_line1'],
             'address_line2' => $input['address_line2'],
             'city' => $input['city'],
             'state_code' => $input['state_code'],
             'county_id' => $input['county_id'],
             'zip' => $input['zip'],
             'contact_number' => $input['contact_number'],
             'logo' => $logoPath,
             'cert_color' => $input['cert_color'],
             'is_active' => $isactive,
             'added_datetime' => now(),
             'added_by' =>Auth::id()
         ];

         if (Client::insert($clientData)) {
             return response()->json(['data' => 'success', 'msg' => 'Client added successfully!']);
         } else {
             return response()->json(['data' => 'error', 'msg' => 'Failed to add client.']);
         }
     }

     public function edit_client(Request $request)
     {
         $clientDetail = Client::with('states', 'counties')->where('id', $request->id)->first();

         return response()->json($clientDetail);
     }


     public function updateClient(Request $request)
     {
         $request->validate([
             'client_name' => 'required',
             'contact_number' => 'nullable',
             'email' => 'nullable|email',
         ]);

         $input = $request->all();
         $isActive = isset($input['is_active']) ? 1 : 0;

         // Check if 'logo' key exists and handle file upload
         $logoPath = null;
         if ($request->hasFile('logo')) {
             $logo = $request->file('logo');
             $logoPath = 'Logo/' . $logo->getClientOriginalName();
             $logo->storeAs('public', $logoPath);
         }

         $clientData = [
             'client_name' => $input['client_name'],
             'client_type_id' => $input['client_type_id'],
             'address_line1' => $input['address_line1'],
             'address_line2' => $input['address_line2'],
             'city' => $input['city'],
             'state_code' => $input['state_code'],
             'county_id' => $input['county_id'],
             'zip' => $input['zip'],
             'cert_color' => $input['cert_color'],
             'contact_number' => $input['contact_number'],
             'is_active' => $isActive,
         ];

         // Add 'logo' to $clientData only if it's provided in the request
         if ($logoPath !== null) {
             $clientData['logo'] = $logoPath;
         }

         $res = Client::where('id', $input['id'])
             ->where('company_id', $input['company_id'])
             ->update($clientData);

         if ($res > 0) {
             return response()->json(['data' => 'success', 'msg' => 'Client Updated Successfully!']);
         } elseif ($res === 0) {
             return response()->json(['data' => 'success', 'msg' => 'No Changes To Update!']);
         } else {
             return response()->json(['data' => 'error', 'msg' => 'Error updating client.']);
         }
     }


     public function clientStatus($id) {

         $client = Client::find($id);
         if ($client->is_active == 1) {
             Client::where('id', $id)->update(['is_active' => 0]);
         } else {
             Client::where('id', $id)->update(['is_active' => 1]);
         }

         return response()->json(['message' => "Status Changed Successfully!"]);
     }


     public function assignmentData(Request $request){
        $user = User::find($request->id);
        $serviceIds = Service::where('is_active', 1)->pluck('id')->toArray();
        $assignedServiceIds = DB::table('user_service_mapping')->where('user_id', $user->id)->where('is_active', 1)->pluck('service_id')->toArray();
        $unassignedServiceIds = array_diff($serviceIds, $assignedServiceIds);

        $unassignedService = Service::whereIn('id', $unassignedServiceIds)->select('id', 'service_name')->get();
        $assignedService = Service::whereIn('id', $assignedServiceIds)->select('id', 'service_name')->get();
        $allService = Service::select('id', 'service_name')->get();

        $response = [
            'unassignedService' => $unassignedService ?? [],
            'assignedService' => $assignedService ?? [],
            'allService' => $allService ?? [],
        ];

        return response()->json($response);
    }

     public function assignService(Request $request) {
        $assignments = $request->input('assignments');
        $dataToInsert = [];

        foreach ($assignments as $assignment) {
            if (!is_null($assignment['service_id']) && !is_null($assignment['user_id'])) {
                $existingAssignment = ServiceUserMapping::where('service_id', $assignment['service_id'])
                    ->where('user_id', $assignment['user_id'])
                    ->first();

                if (!$existingAssignment) {
                    $dataToInsert[] = [
                        'service_id' => $assignment['service_id'],
                        'user_id' => $assignment['user_id'],
                        'is_active' => '1',
                    ];
                } else {
                    ServiceUserMapping::where('service_id', $assignment['service_id'])
                    ->where('user_id', $assignment['user_id'])->update(['is_active' => 1]);
                    $updated = true;
                }
            }
        }
        if (!empty($dataToInsert)) {
            ServiceUserMapping::insert($dataToInsert);
            return response()->json(['data' => 'success', 'msg' => 'Services Assigned Successfully']);
        } else if ($updated) {
            return response()->json(['data' => 'success', 'msg' => 'Services Assigned Successfully']);
        } else {
            return response()->json(['data' => 'error', 'msg' => 'No valid assignments provided'], 400);
        }
    }



    public function removeassignService(Request $request) {
        $assignments = $request->input('assignments');
        foreach ($assignments as $assignment) {
            $service_id = $assignment['service_id'];
            $user_id = $assignment['user_id'];

            $deleteAssignment = ServiceUserMapping::where('service_id', $service_id)
                ->where('user_id', $user_id)
                ->first();
            if ($deleteAssignment) {
                $deleteAssignment->update(['is_active' => 0]);
            }
        }
        return response()->json(['data' => 'success', 'msg' => 'Service Unassinged']);
    }

    public function showUser(Request $request){
        $service_id = $request->id;

        $users = DB::table('users')
            ->join('user_service_mapping', 'users.id', '=', 'user_service_mapping.user_id')
            ->where('user_service_mapping.service_id', $service_id)
            ->where('user_service_mapping.is_active', 1)
            ->select('users.id', 'users.first_name', 'users.last_name')
            ->get();

        return response()->json($users);
    }

    public function updateOrderStatus(Request $request){

        $orderIds = $request->input('orders');
        $isActive = $request->input('is_active');

        Order::whereIn('id', $orderIds)->update(['is_active' => $isActive]);

        $action = $isActive ? 'undeleted' : 'deleted';

        return response()->json(['success' => true, 'message' => "Orders Status Changed Successfully"]);
    }

// Api Configuration
    public function ApiStatus($id) {
        $apiConfigurations = ApiConfig::find($id);

            if ($apiConfigurations) {
                $newStatus = ($apiConfigurations->is_active == 1) ? 0 : 1;
                $apiConfigurations->update(['is_active' => $newStatus]);

                return response()->json(['message' => 'Status Changed Successfully!', 'newStatus' => $newStatus]);
            } else {
                return response()->json(['message' => 'API Configuration not found.'], 404);
            }
        }


        public function getApiConfigurations(Request $request)
        {
            $validatedData = $request->validate([
                'client_id' => 'required',
                'config_data' => 'nullable',
                'is_active' => 'boolean',
            ]);

            $validatedData['is_active'] = $request->has('is_active') ? 1 : 0;
            $validatedData['config_data'] = json_encode($validatedData['config_data']);

            ApiConfig::create($validatedData);

            return response()->json(['success' => true, 'message' => "API Config Inserted Successfully"]);
        }

        public function editApiconfig(Request $request)
        {
            $apiconfigDetail = ApiConfig::where('id', $request->id)->first();

            return response()->json($apiconfigDetail);
        }

        public function updateApiconfig(Request $request)
        {
            $validatedData = $request->validate([
                'id' => 'required',
                'client_id' => 'required',
                'config_data' => 'nullable',
            ]);

            $validatedData['is_active'] = $request->input('is_active') ? 1 : 0;
            ApiConfig::where('id', $validatedData['id'])->update($validatedData);

            return response()->json(['msg' => 'API Config updated successfully']);
        }

}
